#include "DV.hpp"

// loose photon isolation requirement: 1810.05087, page 10
bool is_loose_photon (const HEP::Particle *photon, const HEP::Event* event) {
	//return true;
    double et = 0;
    double pt = 0;
    bool isolated;

    et=sumETIsolation(photon->mom(),event,0.2);
    pt=sumpTisolation(photon->mom(),event,0.2);

    isolated = (et < 0.065 * photon->pT()) && (pt < 0.05 * photon->pT());
    
  
    //isolated = (et < 0.065 * myPhoton->momentum().Et()) &&
    //	(pt < 0.05 * myPhoton->momentum().Et());
    return isolated;
}

void update_vertex_types(vector<dv_candidate> & dv_candidates, bool & found_ee, bool & found_emu, bool & found_mumu)
{
	found_ee=false;
	found_emu=false;
	found_mumu=false;

	for(auto dv: dv_candidates)
	{
		int nlep=dv.N_leptons();
		if(nlep != 2) continue;
		int ne=dv.Nelectrons();
		int nmu=nlep-ne;
		if(ne ==2) found_ee=true;
		else if(nmu == 2) found_mumu=true;
		else found_emu=true;
	}
}



// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 11a
MAdouble64 DVRecoEfficiencyZPrime100GeVToEE (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.1446 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.1308 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*0.2336 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*0.1806 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*0.2486 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*0.1959;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 11b
MAdouble64 DVRecoEfficiencyZPrime100GeVToEMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.3476 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.06565 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.07286 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.1114 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*0.3058 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*0.1688 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*0.3593 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*0.08065;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 11c
MAdouble64 DVRecoEfficiencyZPrime100GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.4065 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.1663 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.1132 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.1372 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*0.3482 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*0.2891 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*0.2159 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*0.1691;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 12a
MAdouble64 DVRecoEfficiencyZPrime250GeVToEE (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.3486 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.2249 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.1143 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.2019 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.3919 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2913 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.09786 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.07478 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*0.4991 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*0.2378 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*0.2396 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*0.1401;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 12b
MAdouble64 DVRecoEfficiencyZPrime250GeVToEMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.2985 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.1634 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.1084 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.0512 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.3463 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2125 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.1317 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.1077 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*0.4617 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*0.3574 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*0.3822 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*0.252;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 12c
MAdouble64 DVRecoEfficiencyZPrime250GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.4706 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.2825 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.1543 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.0892 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.5565 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2735 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.186 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.1727 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*0.584 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*0.409 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*0.2812 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*0.229;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 13a
MAdouble64 DVRecoEfficiencyZPrime500GeVToEE (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.3788 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.2071 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.07406 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.006337 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.3925 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.1871 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.1597 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.1135 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*(pT<400)*0.335 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*(pT<400)*0.2397 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*(pT<400)*0.1814 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*(pT<400)*0.1276 +
	(r_xy>=0)*(r_xy<22)*(pT>=400)*0.7472 +
	(r_xy>=22)*(r_xy<73.)*(pT>=400)*0.6273 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=400)*0.3546 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=400)*0.174;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 13b
MAdouble64 DVRecoEfficiencyZPrime500GeVToEMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.4638 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.2545 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.1186 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.05625 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.496 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2019 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.1605 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.1273 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*(pT<400)*0.4937 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*(pT<400)*0.291 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*(pT<400)*0.1916 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*(pT<400)*0.1859 +
	(r_xy>=0)*(r_xy<22)*(pT>=400)*0.6225 +
	(r_xy>=22)*(r_xy<73.)*(pT>=400)*0.188 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=400)*0.4792 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=400)*0.3304;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 13c
MAdouble64 DVRecoEfficiencyZPrime500GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.4737 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.2965 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.0982 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.1315 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.4909 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2575 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.1685 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.1268 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*(pT<400)*0.4739 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*(pT<400)*0.3037 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*(pT<400)*0.1464 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*(pT<400)*0.1773 +
	(r_xy>=0)*(r_xy<22)*(pT>=400)*0.5953 +
	(r_xy>=22)*(r_xy<73.)*(pT>=400)*0.4242 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=400)*0.1513 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=400)*0.1557;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 14a
MAdouble64 DVRecoEfficiencyZPrime750GeVToEE (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.3997 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.1976 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.05543 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.06085 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.3999 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2041 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.08712 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.07135 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*(pT<400)*0.402 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*(pT<400)*0.2082 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*(pT<400)*0.1151 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*(pT<400)*0.07987 +
	(r_xy>=0)*(r_xy<22)*(pT>=400)*0.5481 +
	(r_xy>=22)*(r_xy<73.)*(pT>=400)*0.2138 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=400)*0.1337 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=400)*0.09805;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 14b
MAdouble64 DVRecoEfficiencyZPrime750GeVToEMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.4912 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.2812 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.06996 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.08305 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.483 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2748 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.1171 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.1321 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*(pT<400)*0.488 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*(pT<400)*0.265 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*(pT<400)*0.1426 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*(pT<400)*0.1754 +
	(r_xy>=0)*(r_xy<22)*(pT>=400)*0.4858 +
	(r_xy>=22)*(r_xy<73.)*(pT>=400)*0.2102 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=400)*0.2005 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=400)*0.117;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 14c
MAdouble64 DVRecoEfficiencyZPrime750GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.4726 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.299 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.06297 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0.1085 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.5027 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2664 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.1506 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.142 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*(pT<400)*0.4899 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*(pT<400)*0.2979 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*(pT<400)*0.1324 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*(pT<400)*0.1427 +
	(r_xy>=0)*(r_xy<22)*(pT>=400)*0.5852 +
	(r_xy>=22)*(r_xy<73.)*(pT>=400)*0.3046 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=400)*0.2431 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=400)*0.2222;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 15a
MAdouble64 DVRecoEfficiencyZPrime1000GeVToEE (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.4282 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.2331 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.09871 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.4022 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.218 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.08756 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.07594 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*(pT<400)*0.3658 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*(pT<400)*0.181 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*(pT<400)*0.1194 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*(pT<400)*0.1011 +
	(r_xy>=0)*(r_xy<22)*(pT>=400)*0.4418 +
	(r_xy>=22)*(r_xy<73.)*(pT>=400)*0.2308 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=400)*0.1847 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=400)*0.09486;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 15b
MAdouble64 DVRecoEfficiencyZPrime1000GeVToEMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.4912 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.3117 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.02943 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.5142 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2636 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.08181 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.06323 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*(pT<400)*0.4907 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*(pT<400)*0.2516 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*(pT<400)*0.162 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*(pT<400)*0.1179 +
	(r_xy>=0)*(r_xy<22)*(pT>=400)*0.5038 +
	(r_xy>=22)*(r_xy<73.)*(pT>=400)*0.1974 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=400)*0.2565 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=400)*0.1011;
}

// https://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/SUSY-2017-04/ figure 15c
MAdouble64 DVRecoEfficiencyZPrime1000GeVToMUMU (MAdouble64 r_xy, MAdouble64 pT) {
    return
	(r_xy>=0)*(r_xy<22)*(pT>=0)*(pT<100)*0.4804 +
	(r_xy>=22)*(r_xy<73.)*(pT>=0)*(pT<100)*0.2951 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=0)*(pT<100)*0.1545 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=0)*(pT<100)*0 +
	(r_xy>=0)*(r_xy<22)*(pT>=100)*(pT<200)*0.4952 +
	(r_xy>=22)*(r_xy<73.)*(pT>=100)*(pT<200)*0.2617 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=100)*(pT<200)*0.1395 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=100)*(pT<200)*0.1051 +
	(r_xy>=0)*(r_xy<22)*(pT>=200)*(pT<400)*0.4971 +
	(r_xy>=22)*(r_xy<73.)*(pT>=200)*(pT<400)*0.272 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=200)*(pT<400)*0.2016 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=200)*(pT<400)*0.1179 +
	(r_xy>=0)*(r_xy<22)*(pT>=400)*0.4823 +
	(r_xy>=22)*(r_xy<73.)*(pT>=400)*0.2521 +
	(r_xy>=73.)*(r_xy<145.)*(pT>=400)*0.1489 +
	(r_xy>=145.)*(r_xy<300.)*(pT>=400)*0.1474;
}

MAdouble64 DVRecoEfficiencyRPVEE (MAdouble64 r_xy, MAdouble64 mDV, MAdouble64 pT) {
    MAdouble64 prob;
    if (r_xy >= 300.)
	prob = 0.;
    else if (r_xy>=0. && r_xy<22.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.1921 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.3477 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.4387 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.4479 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.5141 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.4626 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.4702 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.4634 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.512 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.511 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.5314 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.4693 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.4927 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.5149 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.5082 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.2331 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.5526 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.5384 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.5419 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.568 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0.5635 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.8113 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.6166 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.6327 +
	    (mDV>=788)*(pT>=1000)*0.5751;
    }
    else if (r_xy>=22. && r_xy<38.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.3541 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.194 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.2998 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.2623 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.3207 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.483 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.4336 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.345 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.2982 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.3064 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.4401 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.3068 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.4352 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.3783 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.3347 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.6511 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.4826 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.3751 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.3954 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.3806 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.4554 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.6157 +
	    (mDV>=788)*(pT>=1000)*0.5907;
    }
    else if (r_xy>=38. && r_xy<73.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.2742 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.1038 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.1094 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.04379 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.06766 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.4882 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.297 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.2518 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.1114 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.149 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.3975 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.29 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.3207 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.1904 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.213 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.1063 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.4385 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.3276 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.3596 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.3074 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.1859 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.4721 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.5246 +
	    (mDV>=788)*(pT>=1000)*0.3852;
    }
    else if (r_xy>=73. && r_xy<111.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.2223 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.1001 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.1168 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.1217 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.1034 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.4251 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.2775 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.207 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.1213 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.114 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.2798 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.2581 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.2465 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.1725 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.1702 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.0796 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.3748 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.3246 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.2748 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.2552 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.3635 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.2873 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.2635 +
	    (mDV>=788)*(pT>=1000)*0.2929;
    }
    else if (r_xy>=111. && r_xy<145.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.1134 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.05281 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.06403 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.03169 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.08476 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.1921 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.1502 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.1688 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.09863 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.1215 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.2126 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.2129 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.1652 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.05893 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.09374 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.0551 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.399 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.1939 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.2196 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.1867 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.316 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.2742 +
	    (mDV>=788)*(pT>=1000)*0.1989;
    }
    else if (r_xy>=145. && r_xy<300.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.09885 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.05136 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.06702 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.08401 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.05699 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.1787 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.1596 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.1351 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.09653 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.04982 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.1126 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.1806 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.1485 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.1157 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.08706 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.07383 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.183 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.1651 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.1579 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.1559 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.2574 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.1439 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.1864 +
	    (mDV>=788)*(pT>=1000)*0.159;
    }
    return prob;
}

MAdouble64 DVRecoEfficiencyRPVEMU (MAdouble64 r_xy, MAdouble64 mDV, MAdouble64 pT) {
    MAdouble64 prob;
    if (r_xy >= 300.)
	prob = 0.;
    else if (r_xy>=0. && r_xy<22.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.4368 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.3884 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.4785 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.5599 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.5734 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.4713 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.4845 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.5262 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.554 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.5745 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.5485 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.5287 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.5778 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.599 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.5956 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.4774 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.5855 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.5869 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.5383 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.6107 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*1 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.5438 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.5393 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.6205 +
	    (mDV>=788)*(pT>=1000)*0.5846;
    }
    else if (r_xy>=22. && r_xy<38.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.3352 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.3467 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.3421 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.2102 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.365 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.5769 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.3991 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.4294 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.3986 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.3219 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.1519 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.4513 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.4468 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.4563 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.5208 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.7259 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.4475 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.5134 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.412 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.382 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*1 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.6007 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.6185 +
	    (mDV>=788)*(pT>=1000)*0.4757;
    }
    else if (r_xy>=38. && r_xy<73.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.3313 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.09739 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.1458 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.0325 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.07073 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.4878 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.2875 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.2387 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.1731 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.2268 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.6569 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.3594 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.322 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.2669 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.2327 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.7482 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.5348 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.3979 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.3462 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.3132 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0.8095 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.6291 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.5374 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.4432 +
	    (mDV>=788)*(pT>=1000)*0.3734;
    }
    else if (r_xy>=73. && r_xy<111.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.2545 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.1831 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.136 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.03569 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.05444 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.4414 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.3027 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.1991 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.09753 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.128 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.4675 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.3875 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.2445 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.2412 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.1965 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.5682 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.5104 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.3682 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.2633 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.2726 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0.4829 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.3742 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.2878 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.3613 +
	    (mDV>=788)*(pT>=1000)*0.2635;
    }
    else if (r_xy>=111. && r_xy<145.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.1635 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.1443 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.06005 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.06484 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.2647 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.1805 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.1838 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.07664 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.04723 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.3134 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.3017 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.1771 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.174 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.1692 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.3405 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.2264 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.2729 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.1426 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.1966 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0.4253 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.3718 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.3029 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.3438 +
	    (mDV>=788)*(pT>=1000)*0.2124;
    }
    else if (r_xy>=145. && r_xy<300.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.1601 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.03494 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.07443 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.06943 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.02023 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.2458 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.1628 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.1749 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.1015 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.09633 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.2875 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.2034 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.2098 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.1885 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.126 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.303 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.2675 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.2357 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.1596 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.1628 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0.2828 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.4665 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.2363 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.2426 +
	    (mDV>=788)*(pT>=1000)*0.1873;
    }
    return prob;
}

MAdouble64 DVRecoEfficiencyRPVMUMU (MAdouble64 r_xy, MAdouble64 mDV, MAdouble64 pT) {
    MAdouble64 prob;
    if (r_xy >= 300.)
	prob = 0.;
    else if (r_xy>=0. && r_xy<22.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.4744 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.4833 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.5307 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.5227 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.5446 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.6445 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.5328 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.5531 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.5348 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.565 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.529 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.5174 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.519 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.5498 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.5325 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.1002 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.4996 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.5523 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.5552 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.5454 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.5632 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.571 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.5858 +
	    (mDV>=788)*(pT>=1000)*0.5412;
    }
    else if (r_xy>=22. && r_xy<38.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.5315 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.2824 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.3678 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.303 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.4245 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.5653 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.3727 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.3914 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.3788 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.4224 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.6549 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.3975 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.4227 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.4414 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.3874 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.3313 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.4863 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.429 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.4454 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.4103 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.4721 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.5098 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.493 +
	    (mDV>=788)*(pT>=1000)*0.4332;
    }
    else if (r_xy>=38. && r_xy<73.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.4867 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.1591 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.1479 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.0894 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.1294 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.5294 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.3455 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.2684 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.1434 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.1957 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.4511 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.4409 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.3119 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.2533 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.2074 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.4746 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.459 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.3617 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.2791 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.2877 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.5421 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.3572 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.3233 +
	    (mDV>=788)*(pT>=1000)*0.3614;
    }
    else if (r_xy>=73. && r_xy<111.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.375 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.09754 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.1308 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.1248 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.08723 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.4609 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.2749 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.2154 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.1166 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.1413 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.482 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.3057 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.2237 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.192 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.1838 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.4513 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.4531 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.3413 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.3 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.2192 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0.3609 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.255 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.3015 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.2777 +
	    (mDV>=788)*(pT>=1000)*0.3117;
    }
    else if (r_xy>=111. && r_xy<145.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.2737 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.06655 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.1196 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.03693 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.16 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.3476 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.2063 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.1802 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.05918 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.07751 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.3196 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.2918 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.2388 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.2049 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.2024 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.2961 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.38 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.3238 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.1956 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.2225 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0.1129 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.2521 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.1514 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.2697 +
	    (mDV>=788)*(pT>=1000)*0.1638;
    }
    else if (r_xy>=145. && r_xy<300.) {
	prob = (mDV>=0)*(mDV<62)*(pT>=0)*(pT<200)*0.2564 +
	    (mDV>=62)*(mDV<188)*(pT>=0)*(pT<200)*0.06023 +
	    (mDV>=188)*(mDV<512)*(pT>=0)*(pT<200)*0.09703 +
	    (mDV>=512)*(mDV<788)*(pT>=0)*(pT<200)*0.01542 +
	    (mDV>=788)*(pT>=0)*(pT<200)*0.09108 +
	    (mDV>=0)*(mDV<62)*(pT>=200)*(pT<400)*0.2736 +
	    (mDV>=62)*(mDV<188)*(pT>=200)*(pT<400)*0.2062 +
	    (mDV>=188)*(mDV<512)*(pT>=200)*(pT<400)*0.1842 +
	    (mDV>=512)*(mDV<788)*(pT>=200)*(pT<400)*0.1192 +
	    (mDV>=788)*(pT>=200)*(pT<400)*0.0684 +
	    (mDV>=0)*(mDV<62)*(pT>=400)*(pT<600)*0.2686 +
	    (mDV>=62)*(mDV<188)*(pT>=400)*(pT<600)*0.2033 +
	    (mDV>=188)*(mDV<512)*(pT>=400)*(pT<600)*0.1913 +
	    (mDV>=512)*(mDV<788)*(pT>=400)*(pT<600)*0.1321 +
	    (mDV>=788)*(pT>=400)*(pT<600)*0.1488 +
	    (mDV>=0)*(mDV<62)*(pT>=600)*(pT<1000)*0.2641 +
	    (mDV>=62)*(mDV<188)*(pT>=600)*(pT<1000)*0.226 +
	    (mDV>=188)*(mDV<512)*(pT>=600)*(pT<1000)*0.2237 +
	    (mDV>=512)*(mDV<788)*(pT>=600)*(pT<1000)*0.1939 +
	    (mDV>=788)*(pT>=600)*(pT<1000)*0.1502 +
	    (mDV>=0)*(mDV<62)*(pT>=1000)*0.1348 +
	    (mDV>=62)*(mDV<188)*(pT>=1000)*0.1288 +
	    (mDV>=188)*(mDV<512)*(pT>=1000)*0.2745 +
	    (mDV>=512)*(mDV<788)*(pT>=1000)*0.2434 +
	    (mDV>=788)*(pT>=1000)*0.1588;
    }
    return prob;
}




bool vertexMatch (const HEP::P4& v1, const HEP::P4& v2) {
  double d_space = pow(v1.px() - v2.px(),2) + pow(v1.py() - v2.py(),2) + pow(v1.pz() - v2.pz(),2);
    if (d_space > 1.0) return false; // vertices closer than a mm are combined
    double d_time = fabs(v1.T() - v2.T());
    if (d_time > 1.0E-3)
	return false;
    else
	return true;
}

bool fullMatch(const HEP::Particle &p1, const HEP::Particle &p2)
{
	HEP::P4 x1 = p1.prod_vertex();
	HEP::P4 x2 = p2.prod_vertex();

	// first layer is at 50mm. Assume that if we reconstruct vertices after this point we can see where they come from
	if(x1.pT() > 50.0) return vertexMatch(x1,x2);
	if(fabs(x1.T()-x2.T())>1.0E-3) return false;

	// the rest here is for vertices close to the beam pipe where we have to infer the intersection point. 
	// We should probably do some smearing on the tracks so that we get fake displaced vertices ...

	HEP::P4 v1 = p1.mom();
	HEP::P4 v2 = p2.mom();
	HEP::P4 dvec = v1.cross(v2);

	// check that the minimum distance is less than a mm
	double dist=fabs(dvec.dot3(x1-x2)/dvec.p());

	
	if(dist > 1.0) return false;

	// now check that the *point* of closest approach is before the pixel detector

	double costh = v1.dot3(v2)/v1.p()/v2.p();
	if(fabs(costh)> 0.9999) return false;

	double alphap = (costh*v2.dot3(x1-x2)/v2.p()-v1.dot3(x1-x2)/v1.p())/(1.0-costh*costh);
	HEP::P4 closest_approach = x1 + (alphap/v1.p())*v1;
	if(closest_approach.pT() > 50.0) return false;

	return true;



}




double drCos (HEP::P4 v1,HEP::P4 v2)
{
    double drcos;
    drcos = sqrt(pow(fabs(v1.phi() - v2.phi()) - M_PI,2) + pow(v1.eta() + v2.eta(),2));
    return drcos;
}


bool cosmicRayVeto(vector<HEP::Particle*> &finalStateElectrons, vector<HEP::Particle*> &finalStateMuons)
{
  return crVeto1(finalStateElectrons,0.01) ||
    crVeto1(finalStateMuons,0.01) ||
    crVeto2(finalStateElectrons,finalStateMuons,0.01);
}




double materialVetoWeight(HEP::P4 &vertex) {
    double rxy = vertex.pT();
    double z = vertex.pz();
   double weight;
    weight=
	(z>=-300. && z<300.)*(rxy>=20. && rxy<30.)*0.4354+
	(z>=-300. && z<-200.)*(rxy>=30. && rxy<40.)*0.8915+
	(z>=-200. && z<-100.)*(rxy>=30. && rxy<40.)*0.9043+
	(z>=-100. && z<0.)*(rxy>=30. && rxy<40.)*0.9017+
	(z>=0. && z<100.)*(rxy>=30. && rxy<40.)*0.9049+
	(z>=100. && z<200.)*(rxy>=30. && rxy<40.)*0.9111+
	(z>=200. && z<300.)*(rxy>=30. && rxy<40.)*0.8986+
	(z>=-300. && z<-200.)*(rxy>=40. && rxy<50.)*0.4921+
	(z>=-200. && z<-100.)*(rxy>=40. && rxy<50.)*0.4994+
	(z>=-100. && z<0.)*(rxy>=40. && rxy<50.)*0.4981+
	(z>=0. && z<100.)*(rxy>=40. && rxy<50.)*0.5001+
	(z>=100. && z<200.)*(rxy>=40. && rxy<50.)*0.5003+
	(z>=200. && z<300.)*(rxy>=40. && rxy<50.)*0.4901+
	(z>=-300. && z<-200.)*(rxy>=50. && rxy<60.)*0.7777+
	(z>=-200. && z<-100.)*(rxy>=50. && rxy<60.)*0.7814+
	(z>=-100. && z<0.)*(rxy>=50. && rxy<60.)*0.7477+
	(z>=0. && z<100.)*(rxy>=50. && rxy<60.)*0.7551+
	(z>=100. && z<200.)*(rxy>=50. && rxy<60.)*0.7743+
	(z>=200. && z<300.)*(rxy>=50. && rxy<60.)*0.7795+
	(z>=-300. && z<-200.)*(rxy>=60. && rxy<70.)*0.431+
	(z>=-200. && z<-100.)*(rxy>=60. && rxy<70.)*0.3499+
	(z>=-100. && z<0.)*(rxy>=60. && rxy<70.)*0.3342+
	(z>=0. && z<100.)*(rxy>=60. && rxy<70.)*0.2607+
	(z>=100. && z<200.)*(rxy>=60. && rxy<70.)*0.285+
	(z>=200. && z<300.)*(rxy>=60. && rxy<70.)*0.4353+
	(z>=-300. && z<-200.)*(rxy>=70. && rxy<80.)*0.2951+
	(z>=-200. && z<-100.)*(rxy>=70. && rxy<80.)*0.2946+
	(z>=-100. && z<0.)*(rxy>=70. && rxy<80.)*0.2954+
	(z>=0. && z<200.)*(rxy>=70. && rxy<80.)*0.2944+
	(z>=200. && z<300.)*(rxy>=70. && rxy<80.)*0.2954+
	(z>=-300. && z<-200.)*(rxy>=80. && rxy<90.)*0.4317+
	(z>=-200. && z<-100.)*(rxy>=80. && rxy<90.)*0.4199+
	(z>=-100. && z<0.)*(rxy>=80. && rxy<90.)*0.409+
	(z>=0. && z<100.)*(rxy>=80. && rxy<90.)*0.4201+
	(z>=100. && z<200.)*(rxy>=80. && rxy<90.)*0.4232+
	(z>=200. && z<300.)*(rxy>=80. && rxy<90.)*0.4325+
	(z>=-300. && z<-200.)*(rxy>=90. && rxy<100.)*0.5626+
	(z>=-200. && z<-100.)*(rxy>=90. && rxy<100.)*0.5353+
	(z>=-100. && z<0.)*(rxy>=90. && rxy<100.)*0.4826+
	(z>=0. && z<100.)*(rxy>=90. && rxy<100.)*0.4951+
	(z>=100. && z<200.)*(rxy>=90. && rxy<100.)*0.5078+
	(z>=200. && z<300.)*(rxy>=90. && rxy<100.)*0.5672+
	(z>=-300. && z<-200.)*(rxy>=100. && rxy<110.)*0.5803+
	(z>=-200. && z<-100.)*(rxy>=100. && rxy<110.)*0.5092+
	(z>=-100. && z<0.)*(rxy>=100. && rxy<110.)*0.5027+
	(z>=0. && z<100.)*(rxy>=100. && rxy<110.)*0.4413+
	(z>=100. && z<200.)*(rxy>=100. && rxy<110.)*0.4596+
	(z>=200. && z<300.)*(rxy>=100. && rxy<110.)*0.5765+
	(z>=-300. && z<-200.)*(rxy>=110. && rxy<120.)*0.1911+
	(z>=-200. && z<-100.)*(rxy>=110. && rxy<120.)*0.1893+
	(z>=-100. && z<0.)*(rxy>=110. && rxy<120.)*0.1912+
	(z>=0. && z<100.)*(rxy>=110. && rxy<120.)*0.1942+
	(z>=100. && z<200.)*(rxy>=110. && rxy<120.)*0.1922+
	(z>=200. && z<300.)*(rxy>=110. && rxy<120.)*0.1923+
	(z>=-300. && z<-200.)*(rxy>=120. && rxy<130.)*0.7358+
	(z>=-200. && z<-100.)*(rxy>=120. && rxy<130.)*0.7164+
	(z>=-100. && z<0.)*(rxy>=120. && rxy<130.)*0.7123+
	(z>=0. && z<100.)*(rxy>=120. && rxy<130.)*0.7125+
	(z>=100. && z<200.)*(rxy>=120. && rxy<130.)*0.7366+
	(z>=200. && z<300.)*(rxy>=120. && rxy<130.)*0.744+
	(z>=-300. && z<-200.)*(rxy>=130. && rxy<140.)*0.4384+
	(z>=-200. && z<-100.)*(rxy>=130. && rxy<140.)*0.3614+
	(z>=-100. && z<0.)*(rxy>=130. && rxy<140.)*0.3284+
	(z>=0. && z<100.)*(rxy>=130. && rxy<140.)*0.2881+
	(z>=100. && z<200.)*(rxy>=130. && rxy<140.)*0.3119+
	(z>=200. && z<300.)*(rxy>=130. && rxy<140.)*0.4586+
	(z>=-300. && z<-200.)*(rxy>=140. && rxy<150.)*0.4429+
	(z>=-200. && z<-100.)*(rxy>=140. && rxy<150.)*0.4321+
	(z>=-100. && z<0.)*(rxy>=140. && rxy<150.)*0.4395+
	(z>=0. && z<100.)*(rxy>=140. && rxy<150.)*0.4265+
	(z>=100. && z<200.)*(rxy>=140. && rxy<150.)*0.4283+
	(z>=200. && z<300.)*(rxy>=140. && rxy<150.)*0.4505+
	(z>=-300. && z<300.)*(rxy>=180. && rxy<190.)*0.2526+
	(z>=-300. && z<300.)*(rxy>=190. && rxy<200.)*0.8297+
	(z>=-300. && z<300.)*(rxy>=200. && rxy<210.)*0.7474+
	(z>=-300. && z<300.)*(rxy>=210. && rxy<220.)*0.1934+
	(z>=-300. && z<-200.)*(rxy>=220. && rxy<230.)*0.4929+
	(z>=-200. && z<-100.)*(rxy>=220. && rxy<230.)*0.4329+
	(z>=-100. && z<100.)*(rxy>=220. && rxy<230.)*0.4649+
	(z>=100. && z<200.)*(rxy>=220. && rxy<230.)*0.4329+
	(z>=200. && z<300.)*(rxy>=220. && rxy<230.)*0.4929+
	(z>=-300. && z<-200.)*(rxy>=230. && rxy<240.)*0.64+
	(z>=-200. && z<-100.)*(rxy>=230. && rxy<240.)*0.1+
	(z>=-100. && z<100.)*(rxy>=230. && rxy<240.)*0.388+
	(z>=100. && z<200.)*(rxy>=230. && rxy<240.)*0.1+
	(z>=200. && z<300.)*(rxy>=230. && rxy<240.)*0.64+
	(z>=-300. && z<-200.)*(rxy>=240. && rxy<250.)*0.06+
	(z>=-100. && z<100.)*(rxy>=240. && rxy<250.)*0.032+
	(z>=200. && z<300.)*(rxy>=240. && rxy<250.)*0.06+
	(z>=-300. && z<300.)*(rxy>=250. && rxy<260.)*0.5+
	(z>=-300. && z<-200.)*(rxy>=260. && rxy<270.)*0.1147+
	(z>=-200. && z<200.)*(rxy>=260. && rxy<270.)*0.1+
	(z>=200. && z<300.)*(rxy>=260. && rxy<270.)*0.1147+
	(z>=-300. && z<-200.)*(rxy>=270. && rxy<280.)*0.3129+
	(z>=-200. && z<200.)*(rxy>=270. && rxy<280.)*0.3+
	(z>=200. && z<300.)*(rxy>=270. && rxy<280.)*0.3129+
	(z>=-300. && z<300.)*(rxy>=280. && rxy<290.)*1+
	(z>=-300. && z<300.)*(rxy>=290. && rxy<300.)*1;
    
    return weight;
}

// Returns the fraction of detector volume vetoed for some subvolume in which the
// decay vertex is found. The result will be used to modify the event weight.
double disabledPixelModulesVetoWeight(HEP::P4 &vertex) {
    double rxy = vertex.pT();
    double z = vertex.pz();
    double weight;
    weight =
	(z>=-300. && z<-280.)*(rxy>=20. && rxy<30.)*0.01+
	(z>=-280. && z<-270.)*(rxy>=20. && rxy<30.)*0.0164+
	(z>=-270. && z<-220.)*(rxy>=20. && rxy<30.)*0.018+
	(z>=-220. && z<-210.)*(rxy>=20. && rxy<30.)*0.0132+
	(z>=-210. && z<-160.)*(rxy>=20. && rxy<30.)*0.01+
	(z>=-160. && z<-150.)*(rxy>=20. && rxy<30.)*0.014+
	(z>=-150. && z<-100.)*(rxy>=20. && rxy<30.)*0.02+
	(z>=-100. && z<-90.)*(rxy>=20. && rxy<30.)*0.0224+
	(z>=-90. && z<-40.)*(rxy>=20. && rxy<30.)*0.028+
	(z>=-40. && z<-30.)*(rxy>=20. && rxy<30.)*0.0292+
	(z>=-30. && z<30.)*(rxy>=20. && rxy<30.)*0.04+
	(z>=30. && z<40.)*(rxy>=20. && rxy<30.)*0.004+
	(z>=150. && z<160.)*(rxy>=20. && rxy<30.)*0.0048+
	(z>=160. && z<210.)*(rxy>=20. && rxy<30.)*0.008+
	(z>=210. && z<220.)*(rxy>=20. && rxy<30.)*0.0048+
	(z>=270. && z<280.)*(rxy>=20. && rxy<30.)*0.0056+
	(z>=280. && z<300.)*(rxy>=20. && rxy<30.)*0.028+
	(z>=-300. && z<-280.)*(rxy>=30. && rxy<40.)*0.05+
	(z>=-280. && z<-270.)*(rxy>=30. && rxy<40.)*0.082+
	(z>=-270. && z<-220.)*(rxy>=30. && rxy<40.)*0.09+
	(z>=-220. && z<-210.)*(rxy>=30. && rxy<40.)*0.066+
	(z>=-210. && z<-160.)*(rxy>=30. && rxy<40.)*0.05+
	(z>=-160. && z<-150.)*(rxy>=30. && rxy<40.)*0.07+
	(z>=-150. && z<-100.)*(rxy>=30. && rxy<40.)*0.1+
	(z>=-100. && z<-90.)*(rxy>=30. && rxy<40.)*0.112+
	(z>=-90. && z<-40.)*(rxy>=30. && rxy<40.)*0.14+
	(z>=-40. && z<-30.)*(rxy>=30. && rxy<40.)*0.146+
	(z>=-30. && z<30.)*(rxy>=30. && rxy<40.)*0.2+
	(z>=30. && z<40.)*(rxy>=30. && rxy<40.)*0.02+
	(z>=150. && z<160.)*(rxy>=30. && rxy<40.)*0.024+
	(z>=160. && z<210.)*(rxy>=30. && rxy<40.)*0.04+
	(z>=210. && z<220.)*(rxy>=30. && rxy<40.)*0.024+
	(z>=270. && z<280.)*(rxy>=30. && rxy<40.)*0.028+
	(z>=280. && z<300.)*(rxy>=30. && rxy<40.)*0.14+
	(z>=-300. && z<-280.)*(rxy>=40. && rxy<50.)*0.062+
	(z>=-280. && z<-270.)*(rxy>=40. && rxy<50.)*0.0844+
	(z>=-270. && z<-220.)*(rxy>=40. && rxy<50.)*0.09+
	(z>=-220. && z<-210.)*(rxy>=40. && rxy<50.)*0.078+
	(z>=-210. && z<-160.)*(rxy>=40. && rxy<50.)*0.07+
	(z>=-160. && z<-150.)*(rxy>=40. && rxy<50.)*0.082+
	(z>=-150. && z<-100.)*(rxy>=40. && rxy<50.)*0.1+
	(z>=-100. && z<-90.)*(rxy>=40. && rxy<50.)*0.1156+
	(z>=-90. && z<-40.)*(rxy>=40. && rxy<50.)*0.152+
	(z>=-40. && z<-30.)*(rxy>=40. && rxy<50.)*0.1588+
	(z>=-30. && z<30.)*(rxy>=40. && rxy<50.)*0.22+
	(z>=30. && z<40.)*(rxy>=40. && rxy<50.)*0.0436+
	(z>=40. && z<90.)*(rxy>=40. && rxy<50.)*0.024+
	(z>=90. && z<100.)*(rxy>=40. && rxy<50.)*0.0212+
	(z>=100. && z<150.)*(rxy>=40. && rxy<50.)*0.02+
	(z>=150. && z<160.)*(rxy>=40. && rxy<50.)*0.056+
	(z>=160. && z<210.)*(rxy>=40. && rxy<50.)*0.08+
	(z>=210. && z<220.)*(rxy>=40. && rxy<50.)*0.0528+
	(z>=220. && z<270.)*(rxy>=40. && rxy<50.)*0.012+
	(z>=270. && z<280.)*(rxy>=40. && rxy<50.)*0.0376+
	(z>=280. && z<300.)*(rxy>=40. && rxy<50.)*0.14+
	(z>=-300. && z<-280.)*(rxy>=50. && rxy<60.)*0.08+
	(z>=-280. && z<-270.)*(rxy>=50. && rxy<60.)*0.088+
	(z>=-270. && z<-220.)*(rxy>=50. && rxy<60.)*0.09+
	(z>=-220. && z<-210.)*(rxy>=50. && rxy<60.)*0.096+
	(z>=-210. && z<-100.)*(rxy>=50. && rxy<60.)*0.1+
	(z>=-100. && z<-90.)*(rxy>=50. && rxy<60.)*0.121+
	(z>=-90. && z<-40.)*(rxy>=50. && rxy<60.)*0.17+
	(z>=-40. && z<-30.)*(rxy>=50. && rxy<60.)*0.178+
	(z>=-30. && z<30.)*(rxy>=50. && rxy<60.)*0.25+
	(z>=30. && z<40.)*(rxy>=50. && rxy<60.)*0.079+
	(z>=40. && z<90.)*(rxy>=50. && rxy<60.)*0.06+
	(z>=90. && z<100.)*(rxy>=50. && rxy<60.)*0.053+
	(z>=100. && z<150.)*(rxy>=50. && rxy<60.)*0.05+
	(z>=150. && z<160.)*(rxy>=50. && rxy<60.)*0.104+
	(z>=160. && z<210.)*(rxy>=50. && rxy<60.)*0.14+
	(z>=210. && z<220.)*(rxy>=50. && rxy<60.)*0.096+
	(z>=220. && z<270.)*(rxy>=50. && rxy<60.)*0.03+
	(z>=270. && z<280.)*(rxy>=50. && rxy<60.)*0.052+
	(z>=280. && z<300.)*(rxy>=50. && rxy<60.)*0.14+
	(z>=-300. && z<-280.)*(rxy>=60. && rxy<70.)*0.03+
	(z>=-280. && z<-270.)*(rxy>=60. && rxy<70.)*0.006+
	(z>=-220. && z<-210.)*(rxy>=60. && rxy<70.)*0.03+
	(z>=-210. && z<-160.)*(rxy>=60. && rxy<70.)*0.05+
	(z>=-160. && z<-150.)*(rxy>=60. && rxy<70.)*0.03+
	(z>=-100. && z<-90.)*(rxy>=60. && rxy<70.)*0.009+
	(z>=-90. && z<-40.)*(rxy>=60. && rxy<70.)*0.03+
	(z>=-40. && z<-30.)*(rxy>=60. && rxy<70.)*0.032+
	(z>=-30. && z<30.)*(rxy>=60. && rxy<70.)*0.05+
	(z>=30. && z<40.)*(rxy>=60. && rxy<70.)*0.059+
	(z>=40. && z<90.)*(rxy>=60. && rxy<70.)*0.06+
	(z>=90. && z<100.)*(rxy>=60. && rxy<70.)*0.053+
	(z>=100. && z<150.)*(rxy>=60. && rxy<70.)*0.05+
	(z>=150. && z<160.)*(rxy>=60. && rxy<70.)*0.08+
	(z>=160. && z<210.)*(rxy>=60. && rxy<70.)*0.1+
	(z>=210. && z<220.)*(rxy>=60. && rxy<70.)*0.072+
	(z>=220. && z<270.)*(rxy>=60. && rxy<70.)*0.03+
	(z>=270. && z<280.)*(rxy>=60. && rxy<70.)*0.024+
	(z>=-300. && z<-280.)*(rxy>=70. && rxy<80.)*0.03+
	(z>=-280. && z<-270.)*(rxy>=70. && rxy<80.)*0.006+
	(z>=-220. && z<-210.)*(rxy>=70. && rxy<80.)*0.03+
	(z>=-210. && z<-160.)*(rxy>=70. && rxy<80.)*0.05+
	(z>=-160. && z<-150.)*(rxy>=70. && rxy<80.)*0.03+
	(z>=-100. && z<-90.)*(rxy>=70. && rxy<80.)*0.009+
	(z>=-90. && z<-40.)*(rxy>=70. && rxy<80.)*0.03+
	(z>=-40. && z<-30.)*(rxy>=70. && rxy<80.)*0.032+
	(z>=-30. && z<30.)*(rxy>=70. && rxy<80.)*0.05+
	(z>=30. && z<40.)*(rxy>=70. && rxy<80.)*0.059+
	(z>=40. && z<90.)*(rxy>=70. && rxy<80.)*0.06+
	(z>=90. && z<100.)*(rxy>=70. && rxy<80.)*0.053+
	(z>=100. && z<150.)*(rxy>=70. && rxy<80.)*0.05+
	(z>=150. && z<160.)*(rxy>=70. && rxy<80.)*0.08+
	(z>=160. && z<210.)*(rxy>=70. && rxy<80.)*0.1+
	(z>=210. && z<220.)*(rxy>=70. && rxy<80.)*0.072+
	(z>=220. && z<270.)*(rxy>=70. && rxy<80.)*0.03+
	(z>=270. && z<280.)*(rxy>=70. && rxy<80.)*0.024+
	(z>=-300. && z<-280.)*(rxy>=80. && rxy<90.)*0.16+
	(z>=-280. && z<-270.)*(rxy>=80. && rxy<90.)*0.128+
	(z>=-270. && z<-220.)*(rxy>=80. && rxy<90.)*0.12+
	(z>=-220. && z<-210.)*(rxy>=80. && rxy<90.)*0.15+
	(z>=-210. && z<-160.)*(rxy>=80. && rxy<90.)*0.17+
	(z>=-160. && z<-150.)*(rxy>=80. && rxy<90.)*0.15+
	(z>=-150. && z<-100.)*(rxy>=80. && rxy<90.)*0.12+
	(z>=-100. && z<-90.)*(rxy>=80. && rxy<90.)*0.147+
	(z>=-90. && z<-40.)*(rxy>=80. && rxy<90.)*0.21+
	(z>=-40. && z<-30.)*(rxy>=80. && rxy<90.)*0.214+
	(z>=-30. && z<30.)*(rxy>=80. && rxy<90.)*0.25+
	(z>=30. && z<40.)*(rxy>=80. && rxy<90.)*0.151+
	(z>=40. && z<90.)*(rxy>=80. && rxy<90.)*0.14+
	(z>=90. && z<100.)*(rxy>=80. && rxy<90.)*0.182+
	(z>=100. && z<150.)*(rxy>=80. && rxy<90.)*0.2+
	(z>=150. && z<160.)*(rxy>=80. && rxy<90.)*0.218+
	(z>=160. && z<210.)*(rxy>=80. && rxy<90.)*0.23+
	(z>=210. && z<220.)*(rxy>=80. && rxy<90.)*0.218+
	(z>=220. && z<270.)*(rxy>=80. && rxy<90.)*0.2+
	(z>=270. && z<280.)*(rxy>=80. && rxy<90.)*0.184+
	(z>=280. && z<300.)*(rxy>=80. && rxy<90.)*0.12+
	(z>=-300. && z<-280.)*(rxy>=90. && rxy<100.)*0.16+
	(z>=-280. && z<-270.)*(rxy>=90. && rxy<100.)*0.128+
	(z>=-270. && z<-220.)*(rxy>=90. && rxy<100.)*0.12+
	(z>=-220. && z<-210.)*(rxy>=90. && rxy<100.)*0.15+
	(z>=-210. && z<-160.)*(rxy>=90. && rxy<100.)*0.17+
	(z>=-160. && z<-150.)*(rxy>=90. && rxy<100.)*0.15+
	(z>=-150. && z<-100.)*(rxy>=90. && rxy<100.)*0.12+
	(z>=-100. && z<-90.)*(rxy>=90. && rxy<100.)*0.147+
	(z>=-90. && z<-40.)*(rxy>=90. && rxy<100.)*0.21+
	(z>=-40. && z<-30.)*(rxy>=90. && rxy<100.)*0.214+
	(z>=-30. && z<30.)*(rxy>=90. && rxy<100.)*0.25+
	(z>=30. && z<40.)*(rxy>=90. && rxy<100.)*0.151+
	(z>=40. && z<90.)*(rxy>=90. && rxy<100.)*0.14+
	(z>=90. && z<100.)*(rxy>=90. && rxy<100.)*0.182+
	(z>=100. && z<150.)*(rxy>=90. && rxy<100.)*0.2+
	(z>=150. && z<160.)*(rxy>=90. && rxy<100.)*0.218+
	(z>=160. && z<210.)*(rxy>=90. && rxy<100.)*0.23+
	(z>=210. && z<220.)*(rxy>=90. && rxy<100.)*0.218+
	(z>=220. && z<270.)*(rxy>=90. && rxy<100.)*0.2+
	(z>=270. && z<280.)*(rxy>=90. && rxy<100.)*0.184+
	(z>=280. && z<300.)*(rxy>=90. && rxy<100.)*0.12+
	(z>=-300. && z<-280.)*(rxy>=100. && rxy<110.)*0.13+
	(z>=-280. && z<-270.)*(rxy>=100. && rxy<110.)*0.122+
	(z>=-270. && z<-100.)*(rxy>=100. && rxy<110.)*0.12+
	(z>=-100. && z<-90.)*(rxy>=100. && rxy<110.)*0.138+
	(z>=-90. && z<-40.)*(rxy>=100. && rxy<110.)*0.18+
	(z>=-40. && z<-30.)*(rxy>=100. && rxy<110.)*0.184+
	(z>=-30. && z<30.)*(rxy>=100. && rxy<110.)*0.22+
	(z>=30. && z<40.)*(rxy>=100. && rxy<110.)*0.112+
	(z>=40. && z<90.)*(rxy>=100. && rxy<110.)*0.1+
	(z>=90. && z<100.)*(rxy>=100. && rxy<110.)*0.142+
	(z>=100. && z<150.)*(rxy>=100. && rxy<110.)*0.16+
	(z>=150. && z<160.)*(rxy>=100. && rxy<110.)*0.178+
	(z>=160. && z<210.)*(rxy>=100. && rxy<110.)*0.19+
	(z>=210. && z<220.)*(rxy>=100. && rxy<110.)*0.182+
	(z>=220. && z<270.)*(rxy>=100. && rxy<110.)*0.17+
	(z>=270. && z<280.)*(rxy>=100. && rxy<110.)*0.16+
	(z>=280. && z<300.)*(rxy>=100. && rxy<110.)*0.12+
	(z>=-300. && z<-280.)*(rxy>=110. && rxy<120.)*0.13+
	(z>=-280. && z<-270.)*(rxy>=110. && rxy<120.)*0.122+
	(z>=-270. && z<-100.)*(rxy>=110. && rxy<120.)*0.12+
	(z>=-100. && z<-90.)*(rxy>=110. && rxy<120.)*0.138+
	(z>=-90. && z<-40.)*(rxy>=110. && rxy<120.)*0.18+
	(z>=-40. && z<-30.)*(rxy>=110. && rxy<120.)*0.184+
	(z>=-30. && z<30.)*(rxy>=110. && rxy<120.)*0.22+
	(z>=30. && z<40.)*(rxy>=110. && rxy<120.)*0.112+
	(z>=40. && z<90.)*(rxy>=110. && rxy<120.)*0.1+
	(z>=90. && z<100.)*(rxy>=110. && rxy<120.)*0.142+
	(z>=100. && z<150.)*(rxy>=110. && rxy<120.)*0.16+
	(z>=150. && z<160.)*(rxy>=110. && rxy<120.)*0.178+
	(z>=160. && z<210.)*(rxy>=110. && rxy<120.)*0.19+
	(z>=210. && z<220.)*(rxy>=110. && rxy<120.)*0.182+
	(z>=220. && z<270.)*(rxy>=110. && rxy<120.)*0.17+
	(z>=270. && z<280.)*(rxy>=110. && rxy<120.)*0.16+
	(z>=280. && z<300.)*(rxy>=110. && rxy<120.)*0.12+
	(z>=-300. && z<-280.)*(rxy>=120. && rxy<130.)*0.13+
	(z>=-280. && z<-270.)*(rxy>=120. && rxy<130.)*0.122+
	(z>=-270. && z<-100.)*(rxy>=120. && rxy<130.)*0.12+
	(z>=-100. && z<-90.)*(rxy>=120. && rxy<130.)*0.138+
	(z>=-90. && z<-40.)*(rxy>=120. && rxy<130.)*0.18+
	(z>=-40. && z<-30.)*(rxy>=120. && rxy<130.)*0.184+
	(z>=-30. && z<30.)*(rxy>=120. && rxy<130.)*0.22+
	(z>=30. && z<40.)*(rxy>=120. && rxy<130.)*0.112+
	(z>=40. && z<90.)*(rxy>=120. && rxy<130.)*0.1+
	(z>=90. && z<100.)*(rxy>=120. && rxy<130.)*0.142+
	(z>=100. && z<150.)*(rxy>=120. && rxy<130.)*0.16+
	(z>=150. && z<160.)*(rxy>=120. && rxy<130.)*0.178+
	(z>=160. && z<210.)*(rxy>=120. && rxy<130.)*0.19+
	(z>=210. && z<220.)*(rxy>=120. && rxy<130.)*0.182+
	(z>=220. && z<270.)*(rxy>=120. && rxy<130.)*0.17+
	(z>=270. && z<280.)*(rxy>=120. && rxy<130.)*0.16+
	(z>=280. && z<300.)*(rxy>=120. && rxy<130.)*0.12;
    return weight;
}



void  apply_d_xy_criterion (vector<dv_candidate> &DV_candidates, double d_xy_min) {
    vector<dv_candidate> new_candidates;
    for (int i=0; i<DV_candidates.size(); i++) {
    	if (DV_candidates[i].d_xy() > d_xy_min)
    	    new_candidates.push_back(DV_candidates[i]);
    }
    DV_candidates = new_candidates;
}



/////

void apply_fiducial_volume_criterion (vector<dv_candidate> &DV_candidates, double radius, double half_length) {
    vector<dv_candidate> new_candidates;
    // // remove the DV candidates which have less than two particles
    for (int i=0; i<DV_candidates.size(); i++) {
	double x_DV = DV_candidates[i].get_position().pz();
	double y_DV = DV_candidates[i].get_position().py();
	double z_DV = DV_candidates[i].get_position().pz();
	double r_xy = sqrt(pow(x_DV,2) + pow(y_DV,2));
    	if (fabs(z_DV) < half_length && r_xy < radius)
    	    new_candidates.push_back(DV_candidates[i]);
    }
    DV_candidates = new_candidates;
}

double apply_dis_pixel_mod_veto (vector<dv_candidate> &DV_candidates, std::mt19937 &engine) {
    vector<dv_candidate> new_candidates;
    double weight = 1.;
    for (int i=0; i<DV_candidates.size(); i++) {
	HEP::P4 pos=DV_candidates[i].get_position();
	double w_event = 1.0-disabledPixelModulesVetoWeight(pos);
	if(USE_REWEIGHT) DV_candidates[i].weight_multiplier*=w_event;
	if (w_event != 0) {
		if(USE_REWEIGHT)
		{
			new_candidates.push_back(DV_candidates[i]);
			
		}
		else
		{
			if(rd(engine) < w_event) new_candidates.push_back(DV_candidates[i]);
		}
		weight *= 1.0-w_event;
	}			     
    }
    DV_candidates = new_candidates;
    return 1.0-weight;
}

double apply_material_veto (vector<dv_candidate> &DV_candidates, std::mt19937 &engine) {
    vector<dv_candidate> new_candidates;
    double weight = 1.;
    for (int i=0; i<DV_candidates.size(); i++) {
	if(DV_candidates[i].Nelectrons() < 1) // material veto only applies to electrons
	{
		new_candidates.push_back(DV_candidates[i]);
		weight=0.0; // since we will definitely keep one!!!! 
		continue;
	}
	HEP::P4 pos=DV_candidates[i].get_position();
	//cout << pos.pT() << ", " << pos.pz() << std::endl;
	double w_event = 1.0-materialVetoWeight(pos);
	if(USE_REWEIGHT) DV_candidates[i].weight_multiplier*=w_event;
	if (w_event != 0.0) {

		if(USE_REWEIGHT)
		{
			new_candidates.push_back(DV_candidates[i]);
		}
		else
		{
			if(rd(engine) < w_event) new_candidates.push_back(DV_candidates[i]);
		}
		weight *= 1.0-w_event;	
		    //
	    
	}			     
    }
    DV_candidates = new_candidates;
    return 1.0-weight;
}

void require_leptons (vector<dv_candidate> &DV_candidates, int N_min) {
    vector<dv_candidate> new_candidates;
    for (int i=0; i<DV_candidates.size(); i++)
	if (DV_candidates[i].N_leptons() >= N_min)
	    new_candidates.push_back (DV_candidates[i]);
    DV_candidates = new_candidates;
}

void require_invariant_mass (vector<dv_candidate> &DV_candidates, int m_min) {
    vector<dv_candidate> new_candidates;
    for (int i=0; i<DV_candidates.size(); i++)
	if (DV_candidates[i].get_invariant_mass() > m_min)
	    new_candidates.push_back (DV_candidates[i]);
    DV_candidates = new_candidates;
}


void require_opposite_charge (vector<dv_candidate> &DV_candidates) {

	// here so far we have cheated and just allowed leptons, so we'll just require opposite pids
    vector<dv_candidate> new_candidates;
    for (int i=0; i<DV_candidates.size(); i++) {
	bool pos_charge = false;
	bool neg_charge = false;
	for (int j=0; j<DV_candidates[i].particles.size(); j++) {
	    if (DV_candidates[i].particles[j]->pid() < 0)
		pos_charge = true;
	    else if (DV_candidates[i].particles[j]->pid() > 0)
		neg_charge = true;
	    if (pos_charge && neg_charge) {
		new_candidates.push_back(DV_candidates[i]);
		break;
	    }
	}
    }
    DV_candidates = new_candidates;
}

/////

void apply_trigger_matching (vector<dv_candidate> &DV_candidates) {
    vector<dv_candidate> new_candidates;
    for (int i=0; i<DV_candidates.size(); i++) {
	int n_single_photon = 0;
	int n_di_photon = 0;
	for (int j=0; j<DV_candidates[i].particles.size(); j++) {
	    int pdgid = DV_candidates[i].particles[j]->pid();
	    double pt = DV_candidates[i].particles[j]->pT();
	    double abseta = fabs(DV_candidates[i].particles[j]->eta());
	    // electron
	    if (fabs(pdgid) == 11) {
		if (abseta < 2.47) {
		    if (pt > 140.) n_single_photon++;
		    if (pt > 50.) n_di_photon++;
		}
	    }
	    // muon
	    if (fabs(pdgid) == 13) {
		//double absd0 = fabs(DV_candidates[i].particles[j]->d0());
		double absd0 = d0Calc(DV_candidates[i].particles[j]->prod_vertex(),DV_candidates[i].particles[j]->mom());


		if (absd0 > 1.5 && pt > 60 && abseta < 1.05)
		    DV_candidates[i].muon_trigger = true;
	    }
	}
	if (n_single_photon >= 1)
	    DV_candidates[i].single_photon_trigger = true;
	if (n_di_photon >= 2)
	    DV_candidates[i].di_photon_trigger = true;
	if (DV_candidates[i].single_photon_trigger || DV_candidates[i].di_photon_trigger ||
	    DV_candidates[i].muon_trigger)
	    new_candidates.push_back(DV_candidates[i]);
    }
    DV_candidates = new_candidates;
}


void apply_preselection_matching (vector<dv_candidate> &DV_candidates) {
    vector<dv_candidate> new_candidates;
    
    for (int i=0; i<DV_candidates.size(); i++) {
	// counters for preselection requirements
	int NMuons62GeV = 0; // [ pt>62GeV and |eta|<1.07 and |d0| > 1.5 mm ]
	int NMuons10GeV = 0; // [ pt>10GeV and |eta|<2.5  and |d0| > 1.5 mm ]
	int NElectrons150GeV = 0;     // [ pt>150GeV         and |eta|<2.5 and |d0|>2.0  ]
	int NElectrons55GeVDispl = 0; // [ 150GeV>=pt>55GeV  and |eta|<2.5 and |d0|>2.0  ]
	int NElectrons55GeV = 0;      // [ 150GeV>=pt>55GeV  and |eta|<2.5 and |d0|<=2.0 ]
	int NElectrons10GeV = 0;      // [ 55GeV>=pt>10GeV   and |eta|<2.5 and |d0|>2.0  ]
	int NPhotons150GeV = 0;
	int NPhotons55GeV = 0;
	int NPhotons10GeV = 0;

	for (int j=0; j<DV_candidates[i].particles.size(); j++) {
	    int pdgid = DV_candidates[i].particles[j]->pid();
	    double pt = DV_candidates[i].particles[j]->pT();
	    double abseta = fabs(DV_candidates[i].particles[j]->eta());
	    //MAdouble64 absd0 = fabs(DV_candidates[i].particles[j]->d0());
	    double absd0 = d0Calc(DV_candidates[i].particles[j]->prod_vertex(),DV_candidates[i].particles[j]->mom());
	    // electron
	    if (fabs(pdgid) == 11) {
		if (abseta < 2.5) {
		    if (absd0 > 2.0) {
			if (pt > 150.) NElectrons150GeV++;
			else if (pt > 55.) NElectrons55GeVDispl++;
			else if (pt > 10.) NElectrons10GeV++;
		    }
		    else if (pt > 55.) NElectrons55GeV++;
		}
	    }
	    // muon
	    if (fabs(pdgid) == 13) {
		if (absd0 > 1.5) {
		    if (pt > 62. && abseta < 1.07) NMuons62GeV++;
		    if (pt > 10. && abseta < 2.5) NMuons10GeV++;
		}
	    }
	}
	// check preselection cuts
	bool preselected = false;
	if (DV_candidates[i].muon_trigger) {
	    if (NMuons62GeV >= 1) preselected = true;
	}
	if (DV_candidates[i].single_photon_trigger) {
	    // single photon line 5
	    if (NElectrons150GeV >= 1) preselected = true;
	}
	if (DV_candidates[i].di_photon_trigger) {
	    // diphoton line 7
	    if ((NElectrons150GeV + NElectrons55GeVDispl) >= 2)
		preselected = true;
	    // diphoton lines 8+9
	    else if ((NElectrons150GeV + NElectrons55GeVDispl + NElectrons55GeV) >=1)
		preselected = true;
	}
	if (preselected)
	    new_candidates.push_back(DV_candidates[i]);
	// else {
	//     cout << "\n\nPreselection matching...\n";
	//     for (int j=0; j<DV_candidates[i].particles.size(); j++) {	    
	// 	int pdgid = DV_candidates[i].particles[j]->mc()->pdgid();
	// 	MAdouble64 pt = DV_candidates[i].particles[j]->pt();
	// 	MAdouble64 abseta = fabs(DV_candidates[i].particles[j]->eta());
	// 	MAdouble64 absd0 = fabs(DV_candidates[i].particles[j]->d0());
	    

	// 	cout << "pdgid:\t" << pdgid << endl;
	// 	cout << "pt:\t" << pt << endl;
	// 	cout << "abseta:\t" << abseta << endl;
	// 	cout << "absd0:\t" << absd0 << endl << endl;
	//     }
	// }
    }
    DV_candidates = new_candidates;
}





